#include <math.h>
#include <stdio.h>
#include <time.h>
#include <limits.h>

extern void initialise (void);
extern void base_loop (void);
extern void peak_loop (void);
extern void preload (void);
extern const char *compare_loops (void);

extern int base_repeat_count;

#define BASE_FACTOR 250
#define ITERATIONS (BASE_FACTOR * base_repeat_count)
#define REPEATS 40

#define FLOAT_EPSILON 0x1.0p-6f

#define TIMER CLOCK_PROCESS_CPUTIME_ID

int
floats_eq_p (float a, float b)
{
  if (a == b)
    return 1;
  else
    return fabs (a - b) < FLOAT_EPSILON;
}

static void
one_iteration (int i)
{
  while (i-- > 0)
    peak_loop ();
}

static void
display_results (double *times)
{
  double quickest, mean, sum, sum_of_squared_differences, std_dev;
  int i;

  quickest = times[0];
  for (i = 1 ; i < REPEATS; i++)
    if (quickest > times[i])
      quickest = times[i];

  sum = 0.0;
  for (i = 0 ; i < REPEATS; i++)
    sum += times[i];
  mean = sum / REPEATS;  

  sum_of_squared_differences = 0.0;
  for (i = 0; i < REPEATS ; i++)
    sum_of_squared_differences += (times[i] - mean) * (times[i] - mean);
  std_dev = sqrt (sum_of_squared_differences / REPEATS);

  printf ("%d runs take %gms, mean = %gms, std dev = %gms\n",
	  ITERATIONS, quickest * 1000.0, mean * 1000.0, std_dev * 1000.0);
}

int
main (void)
{
  int i;
  struct timespec before, after;
  double times[REPEATS];
  const char *bad_var;

  initialise ();
  base_loop ();
  peak_loop ();
  bad_var = compare_loops ();
  if (bad_var)
    {
      printf ("%s has the wrong contents\n", bad_var);
      return 1;
    }

  for (i = 0; i < REPEATS; i++)
    {
      initialise ();
      preload ();
      one_iteration (1);
      clock_gettime (TIMER, &before);
      preload ();
      one_iteration (ITERATIONS);
      clock_gettime (TIMER, &after);
      times[i] = ((after.tv_sec - before.tv_sec) * 1.0
		  + (after.tv_nsec - before.tv_nsec) * 1.0e-9);
    }
  display_results (times);
  return 0;
}
